/*
 *  subtitle editor
 *  Copyright @ 2005-2006, IDJAAD Djamel (aka Kitone)
 *
 *  Contact: kitone at free point fr
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public
 *  License along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *  See gpl.txt for more information regarding the GNU General Public License.
 *
 *
 *  \file
 *  \brief 
 *  \author IDJAAD Djamel (kitone at free point fr)
 */

#include "utility.h"
#include "DialogFileChooser.h"
#include "Config.h"
#include "Encodings.h"
#include <libglademm/xml.h>

class ColumnEncoding : public Gtk::TreeModel::ColumnRecord
{
public:
	ColumnEncoding()
	{
		add(use);
		add(name);
		add(charset);
	}
public:
  Gtk::TreeModelColumn<bool>					use;
  Gtk::TreeModelColumn<Glib::ustring>	name;
  Gtk::TreeModelColumn<Glib::ustring>	charset;
};


class EncodingsSelector : public Gtk::HBox
{
public:
	EncodingsSelector()
	:Gtk::HBox(false, 3), 
		m_labelEncodings(_("Character Coding:"), 1.0,0.5)
	{
		m_buttonEncodingsMng.add(*manage(new Gtk::Image(Gtk::Stock::ADD, Gtk::ICON_SIZE_BUTTON)));
	
		pack_start(m_labelEncodings, false,false);
		pack_start(m_comboEncodings,true,true);
		pack_end(m_buttonEncodingsMng,false,false);

		m_buttonEncodingsMng.signal_clicked().connect(
				sigc::mem_fun(*this, &EncodingsSelector::open_encodgins_manager));


		//
		init_combo_encodgins();
		
		show_all();
	}

	void init_combo_encodgins()
	{
		m_comboEncodings.clear();
		// TODO : use clear_items for GTK+ 2.8
		//m_comboEncodings.clear_items();

		m_comboEncodings.append_text("Auto Detected");

		Config &cfg = Config::getInstance();

		std::list<Glib::ustring> list_encodgins;
		if(cfg.get_value_string_list("encodings", "encodings", list_encodgins))
		{
			std::list<Glib::ustring>::const_iterator it;
			for(it = list_encodgins.begin(); it!=list_encodgins.end(); ++it)
			{
				EncodingInfo *info= Encodings::get_from_charset(*it);
				if(info)
				{
					gchar *name = g_strdup_printf("%s (%s)", info->name, info->charset);		
					m_comboEncodings.append_text(name);
					g_free(name);
				}
			}
		}
		/*
		for(unsigned int i=0; encodings_info[i].charset != NULL; ++i)
		{
			gchar *name = g_strdup_printf("%s (%s)", encodings_info[i].name, encodings_info[i].charset);		
			m_comboEncodings.append_text(name);
			g_free(name);
		}
		*/
		// init default selection
		Glib::ustring default_coding;
		if(cfg.get_value_string("encodings", "default", default_coding))
		{
			EncodingInfo *info= Encodings::get_from_charset(default_coding);
			if(info)
			{
				gchar *name = g_strdup_printf("%s (%s)", info->name, info->charset);		
				m_comboEncodings.set_active_text(name);
				g_free(name);
			}
			else
				m_comboEncodings.set_active_text("Auto Detected");
		}
		else
			m_comboEncodings.set_active_text("Auto Detected");
	}
	
	void open_encodgins_manager()
	{
		m_dialog = NULL;
		m_treeviewEncodings = NULL;
		
		Glib::RefPtr<Gnome::Glade::Xml> refXml = Gnome::Glade::Xml::create(get_share_dir("dialog-encodings-chooser.glade"));
	
		refXml->get_widget("dialog-encodings-chooser", m_dialog);
		refXml->get_widget("treeviewEncodings", m_treeviewEncodings);

		// creation du treeview
		ColumnEncoding m_column;
	
		Glib::RefPtr<Gtk::ListStore> list = Gtk::ListStore::create(m_column);

		m_treeviewEncodings->set_model(list);

	
		Gtk::TreeViewColumn* column = NULL;
		Gtk::CellRendererToggle* toggle = NULL;
		Gtk::CellRendererText* name = NULL;
		Gtk::CellRendererText* charset = NULL;

		//
		column = manage(new Gtk::TreeViewColumn());
		m_treeviewEncodings->append_column(*column);
	
		toggle = manage(new Gtk::CellRendererToggle);
	
		column->pack_start(*toggle, false);
		column->add_attribute(toggle->property_active(), m_column.use);

		//
		column = manage(new Gtk::TreeViewColumn(_("Name")));
		m_treeviewEncodings->append_column(*column);	

		name = manage(new Gtk::CellRendererText);
		column->pack_start(*name, false);
		column->add_attribute(name->property_text(), m_column.name);

		//
		column = manage(new Gtk::TreeViewColumn(_("Charset")));
		m_treeviewEncodings->append_column(*column);
	
		charset = manage(new Gtk::CellRendererText);

		column->pack_start(*charset, false);
		column->add_attribute(charset->property_text(), m_column.charset);
	

		for(unsigned int i=0; encodings_info[i].charset != NULL; ++i)
		{
			Gtk::TreeIter it = list->append();

			(*it)[m_column.name] = encodings_info[i].name;
			(*it)[m_column.charset] = encodings_info[i].charset;
		}
		
		// signal for check button
		toggle->signal_toggled().connect(
				sigc::mem_fun(*this, &EncodingsSelector::encoding_toggled));

		
		load_config_encodings();

		//
		if(m_dialog->run() == Gtk::RESPONSE_OK)
		{
			Config &cfg = Config::getInstance();
			//cfg.remove_key("encodings", "encodings");
			
			std::list<Glib::ustring> list;
			
			Gtk::TreeNodeChildren rows = m_treeviewEncodings->get_model()->children();
			for(Gtk::TreeIter iter = rows.begin(); iter; ++iter)
			{
				if((*iter)[m_column.use])
					list.push_back((*iter)[m_column.charset]);
			}
			
			cfg.set_value_string_list("encodings", "encodings", list);

			init_combo_encodgins();
		}

		delete m_dialog;
	}

	void encoding_toggled(const Glib::ustring &_path)
	{
		Gtk::TreePath path(_path);

		Gtk::TreeIter iter = m_treeviewEncodings->get_model()->get_iter(path);

		ColumnEncoding m_column;
		(*iter)[m_column.use] = ! (*iter)[m_column.use];
	}

	void load_config_encodings()
	{
		Config &cfg = Config::getInstance();

		// encodings
		std::list<Glib::ustring> list;
		if(cfg.get_value_string_list("encodings", "encodings", list))
		{
			ColumnEncoding m_column;
		
			std::list<Glib::ustring>::const_iterator it;
			for(it=list.begin(); it!=list.end(); ++it)
			{
				Gtk::TreeNodeChildren rows = m_treeviewEncodings->get_model()->children();
				for(Gtk::TreeIter iter = rows.begin(); iter; ++iter)
				{
					if((*iter)[m_column.charset] == *it)
					{
						(*iter)[m_column.use] = true;
						break;
					}
				}
			}
		}
	}

	Glib::ustring getEncoding()
	{
		Glib::ustring text = m_comboEncodings.get_active_text();
		
		Glib::ustring::size_type a = text.find('(');
		Glib::ustring::size_type b = text.find(')', a);

		if(a == Glib::ustring::npos || b == Glib::ustring::npos)
			return "";
		
		return text.substr(a+1,b-a-1);
	}
protected:
	Gtk::Label				m_labelEncodings;
	Gtk::ComboBoxText m_comboEncodings;
	Gtk::Button				m_buttonEncodingsMng;
	
	Gtk::Dialog*			m_dialog;
	Gtk::TreeView*		m_treeviewEncodings;
};





/*
 *
 */
DialogFileChooser::DialogFileChooser(const Glib::ustring &title, Gtk::FileChooserAction action)
:Gtk::FileChooserDialog(title, action)
{
  add_button(Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL);
  add_button(Gtk::Stock::OK, Gtk::RESPONSE_OK);
	set_default_response(Gtk::RESPONSE_OK);

	m_encodingsSelector = manage(new EncodingsSelector);
	set_extra_widget(*m_encodingsSelector);
}

/*
 *
 */
Glib::ustring DialogFileChooser::getEncoding()
{
	Glib::ustring text = m_encodingsSelector->getEncoding();
	if(text == "Auto Detected")
		return "";
	
	return text;
}


/*
 * exemple :
 * name = "All Format Supported"
 * patterns = "*.ssa;*.ass;*.sub;*.txt"
 */
void DialogFileChooser::addFilter(const Glib::ustring &name, const Glib::ustring &patterns)
{
	Gtk::FileFilter filter;
	
	filter.set_name(name);
	
	std::istringstream iss(patterns);
	std::string word;
	while( std::getline(iss, word, ';') )
	{
		filter.add_pattern(word);
	}

	add_filter(filter);
}

